<?php
/* Copyright 2007-2009 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
class AppController extends Controller {
	
	/**List of actions that will trigger prepareSmartFlash().
	 * Defaults to add, edit, delete.
	 *
	 * @var array
	 * @see prepareSmartFlash()
	 */
	protected $smartFlashActions = array('add', 'edit', 'delete');
	
	protected function error($title, $message, $skipSanitize = true) {
		$this->cakeError('error', array(array('message' => $message, 'base' => '', 'code' => '', 'name' => $title, 'skipSanitize' => $skipSanitize)));
		return false;
	}
	
	protected function errorNoSuchEntry($message) {
		header("HTTP/1.0 404 Not Found");
		$this->cakeError('error',
			array('code' => '',
				'name' => __('Not found', true),
				'message' => $message
			)
		);
	}
	
	/** Returns a list of filter => value, based on the names from the first
	 * parameter and fetched either from URL parameters or POST data.
	 * This method has no effect on the object state.
	 *
	 * @param array $names the accepted parameters (other URL parameters are
	 * ignored)
	 * @return array An array of parameter => value, with missing or empty
	 * parameters marked as null.
	 */
	protected function getFilters($names) {
		$values = array();
		
		foreach ($names as $parameter) {
			$val = null;
			
			if (isset($this->params['named'][$parameter]))
				$val = $this->params['named'][$parameter];
			else if (isset($this->data[$parameter]))
				$val = $this->data[$parameter];
			
			if (empty($val) || $val === '0')
				$val = null;
			
			$values[$parameter] = $val;
		}
		
		return $values;
	}
	
	/** Prepares pagination on a custom set of data (i.e. not returned by
	 * paginate()).
	 *
	 * @param integer $page The current page number
	 * @param integer $limit The number of items to show per page
	 * @param integer $totalItems The total number of items in the non-paginated
	 * set
	 * @param array $params An optionnal array of URL parameters to pass from
	 * one page to the other
	 */
	protected function prepareManualPagination($model, $page, $limit, $totalItems, $params = array()) {
		$this->set('doPaginate', true);
		$this->params['paging'][$model]['page'] = $page;
		if (isset($this->params['named']['sort']))
			$params['sort'] = $this->params['named']['sort'];
		if (isset($this->params['named']['direction']))
			$params['direction'] = $this->params['named']['direction'];
		$this->params['paging'][$model]['pageCount'] = ceil($totalItems/$limit);
		$this->params['paging'][$model]['prevPage'] = $page > 1;
		$this->params['paging'][$model]['nextPage'] = $page < $this->params['paging'][$model]['pageCount'];
		$this->params['paging'][$model]['last'] = ceil($totalItems/$limit);
		$this->params['paging'][$model]['defaults'] = array('page' => 1, 'limit' => $limit);
		$this->params['paging'][$model]['options'] = array_merge($params, array('page' => $page, 'limit' => $limit));
	}
	
	/**
 	 * Called before the controller action. Overridden in subclasses.
 	 * Be sure to call parent::beforeFilter() when overriding this.
 	 */
	function beforeFilter() {
		$this->prepareSmartFlash();
	}	
	
	/* Note to self: working with session isn't safe:
	 * - User browses series A
	 * - User edits album A.1 in a new tab (-> referer set to series B)
	 * - User browses series B
	 * - User edits album B.1 (-> referer set to series B)
	 * - User saves and is redirected to series B
	 * - User goes back to second tab, saves, and is redirected to series B
	 *   because the reference to A was overriden.
	 * 
	 * We cannot work with a stack of redirects either because there is no
	 * guarantee that they will be popped.
	 */
	/** Prepares a view variable for smart flash if the current action
	 * is present in the $smartFlashActions array.
	 *
	 * @param boolean $force true if the variable should be prepared
	 * even if the action isn't mentionned in $smartRedirectActions
	 */
	protected function prepareSmartFlash($force = false) {
		if (($force || in_array($this->params['action'], $this->smartFlashActions))
				&& isset($_SERVER['HTTP_REFERER'])
				&& $_SERVER['HTTP_REFERER'] !== Router::url('/', true))
			$this->set('smartFlash', $_SERVER['HTTP_REFERER']);
		else
			$this->set('smartFlash', '');
	}
	
	/** Flashes a messages and goes to the default URL, unless the action is
	 * candidate for smart redirection, in which case defaultURL is ignored
	 * and the user is redirected from where he was two steps ago.
	 *
	 * @param string $defaultURL
	 * @param string $message
	 */
	protected function smartFlash($message, $defaultURL) {
		if (! empty($this->params['form']['smartFlash']))
			$this->flash($message, $this->params['form']['smartFlash']);
		else
			$this->flash($message, $defaultURL);
	}
}
?>