<?php
/* Copyright 2007-2009 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
class AuthorsController extends AppController {
	public $scaffold;
	public $uses = array('Author', 'Artist', 'Colorist', 'Derivative', 'Writer');
	public $helpers = array('Html', 'List', 'Menu', 'MyPaginator', 'Quicksearch', 'Shadowbox');
	public $paginate = array('limit' => 42, 'page' => 1, 'order'=>array('name' => 'asc'), 'fields' => array('id','name', 'fname'), 'recursive' => 0); 

	function index() {
		$this->paginate['limit'] = ($perColumn = Configure::read('Demyo.itemsPerIndexColumns')) * Configure::read('Demyo.columnsPerIndexPage');
		$this->set('perColumn', $perColumn);
		
		if (isset($this->params['named']['startsWith'])) {
			$this->paginate['page'] = $this->Author->findPageNumberStartsWith('name', $this->params['named']['startsWith'], $this->paginate['limit']);
		}
		
		$this->set('authors', $this->paginate('Author'));
	}
	
	function view($id = null) {
		$this->Author->id = $id;
		$this->Author->recursive = 0;
		if (! $author = $this->Author->read())
			$this->errorNoSuchEntry(__('This author does not exist.', true));
		$author = current($author);
		$author['derivative_count'] = $this->Derivative->getDerivativesCount('Artist', $id);
		$this->set('author', $author);
		
		foreach (array('Artist', 'Writer', 'Colorist') as $model) {
			$this->$model->recursive = 2;
			$this->$model->contain(array(
				'Album.id', 'Album.cycle', 'Album.number', 'Album.title',
				'Album.Series.name'
			));
			$result = $this->$model->find(array('id' => $id), array('id'));
			usort($result['Album'], array('Album', 'sortWithSeries'));
			$this->set("as$model", $result['Album']);
		}
    }
	
	private function getSeries($album_list) {
		$series = array();
		foreach ($album_list as $album)
			if (! isset($series[$album['series_id']]))
				$series[$album['series_id']] = $album['Series'];
		return $series;
	}
	
	function add() {
		if (!empty($this->data)) {
			if ($this->Author->save($this->data)) {
				$this->smartFlash(__('The author has been added', true),
					'/authors/view/'.$this->Author->id);
			}
		}
		// else just show the form
	}
	
	function edit($id = null) {
		if (empty($this->data)) {
			$this->Author->id = $id;
			$this->Author->unbindAll();
			if (! $this->data = $this->Author->read())
				$this->errorNoSuchEntry(__('This author does not exist.', true));
		}
		
		else {
			if ($this->Author->save($this->data)) {
				$this->smartFlash(__('The author has been updated', true),
					'/authors/view/'.$this->data['Author']['id']);
			}
		}
	}
}

?>
