<?php
/* Copyright 2007-2009 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
class ImagesController extends AppController {
	public $scaffold;
	public $helpers = array('HelpBrowser', 'jQuery', 'List', 'Menu', 'MyPaginator', 'Quicksearch', 'Shadowbox');
	public $components = array('File', 'PhpThumb');
	public $uses = array('Image', 'Album', 'Series');
	
	public function index() {
		$images = array();
		
		$this->set('doPaginate', false);

		if (isset($this->params['named']['fromSeries']))
			$images = $this->Series->getImages($this->params['named']['fromSeries']);
		else if (isset($this->params['named']['fromAlbum']))
			$images = $this->Album->getImages($this->params['named']['fromAlbum']);
		else {
			$limit = Configure::read('Demyo.Gallery_thumbsPerPage');
			if (isset($this->params['named']['startsWith']))
				$page = $this->Image->findPageNumberStartsWith('description', $this->params['named']['startsWith'], $limit);
			else
				$page = isset($this->params['named']['page'])?$this->params['named']['page']:1;
			
			$images = Set::extract($this->Image->find('all',
					array('limit' => $limit, 'page' => $page, 'recursive' => -1)),
				'{n}.Image');
			
			// Prepare paginator
			$this->prepareManualPagination('Image', $page, $limit, $this->Image->find('count'));
		}
		
		foreach($images as &$image)
			$image['thumb_url'] = $this->PhpThumb->getThumbnail(IMAGES.$image['url']);
		
		$this->set('images', $images);
	}
	
	public function view($id = null) {
		if ($id === null)
			$this->redirect('/images/');
		
		$this->Image->id = $id;
		$this->Image->recursive = 2;
		$this->Image->contain(
			array(
			'Album.id', 'Album.title',
			'Derivative.id', 'Derivative.Source',
				'Derivative.DerivativeType.name',
				'Derivative.Series.name'));
		if (! $image = $this->Image->read())
			$this->errorNoSuchEntry(__('This image does not exist.', true));
		$image['Image']['thumb_url'] = $this->PhpThumb->getThumbnail(IMAGES.$image['Image']['url']);
		$this->set('image', $image['Image']);
		$this->set('albums', $image['Album']);
		$this->set('derivatives', $image['Derivative']);
	}
	
	public function add() {
		if (!empty($this->data)) {
			$this->data['Image']['is_local'] = (preg_match('@^(http|https|ftp)://.*@', $this->data['Image']['url']) === 0);
			
			if ($this->Image->save($this->data)) {
				$this->smartFlash(__('The image has been added', true),
					'/images/view/'.$this->Image->id);
			}
		}
	}
	
	public function edit($id = null) {
		if (empty($this->data)) {
			$this->Image->id = $id;
			if (! $this->data = $this->Image->read())
				$this->errorNoSuchEntry(__('This image does not exist.', true));
		}
		
		else {
			$this->data['Image']['is_local'] = (preg_match('@^(http|https|ftp)://.*@', $this->data['Image']['url']) === 0);
			
			if ($this->Image->save($this->data)) {
				$this->smartFlash(__('The image has been updated', true),
					'/images/view/'.$this->data['Image']['id']);
			}
		}
	}
	
	public function browser() {
		if (!isset($this->data['Browser']['path'])) {
			$this->set('isAjax', false);
			$this->layout = 'simple';
			
			if (!empty($this->params['named']['base'])) {
				$url = $this->Image->find('first', array('fields' => 'url',
					'conditions' => array('id' => $this->params['named']['base']), 'recursive' => -1));
				$this->set('baseDir', dirname(
					preg_replace('+^'.COLLECTION_IMAGES_DIR.'/+', '', $url['Image']['url'])));
			} else {
				$this->set('baseDir', '');
			}
		}
		
		else {
			$this->set('isAjax', true);
			$this->layout = 'raw';
			$path = $this->data['Browser']['path'];
			
			if (empty($path)) {
				$path = COLLECTION_IMAGES;
				$this->set('base', $path);
			} else {
				$path = preg_replace('@/?[^/]+/\.\.@', '', $path); // 'foo/bar/..' => 'foo/'
				$this->set('base', $path);
				
				// Received path is a filtered one [1], prepend it with base directory
				$path = COLLECTION_IMAGES.$path;
			}			
						
			$dirRelation = $this->File->dir_cmp(COLLECTION_IMAGES, $path);
			
			if ((! is_dir($path))
					|| ($dirRelation < 0) || $dirRelation === false)
				return $this->error(__('Images', true), __('Invalid path', true));
			
			
			$list = $this->File->getFileList($path, '/^.+\.(jpg|gif|png|bmp)$/i');
			if ($list == null) 
				return $this->error(__('Images', true), __('Failed to retreive file list', true));
			
			$this->set('isBelowRoot', $dirRelation != 0);
			
			// [1] Strip base path from the list, so that the client doesn't have
			// too much info on the server filesystem.
			foreach ($list['dirs'] as &$dir)
				$dir = str_replace(COLLECTION_IMAGES, '', $dir);
			foreach ($list['files'] as &$file)
				$file = str_replace(COLLECTION_IMAGES, '', $file);
			$this->set('fileList', $list);
		}
	}
}

?>
