<?php
/* Copyright 2007-2009 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */

class ListHelper extends AppHelper {
	public $helpers = array('Html', 'jQuery');

	/** Returns a comma separated list of values
	 * @param items An array of records
	 * @param keys An ordered array of the keys in the record that should be
	 * used to define the value. Default to the 'name' key only.
	 * @param base_url An url to view the values. Values will be linked
	 * to that url, where %d will be replaced with the value indexed by 'id'
	 * in the record. Set to null for no linking.
	 */
	public function CSVList($items, $keys = array('name'), $base_url = null) {
		$i = 0;
		$num_items = sizeof($items);
		$num_keys = sizeof($keys);
		$CSVList = '';
		
		foreach ($items as $item) {
			// Build text to print, space separated
			$j = 0;
			$title = '';
			foreach ($keys as $key) {
				$title .= $item[$key];
				if ($j++ < $num_keys-1)
					$title .= ' ';
			}
			
			// Link if required
			if (!empty($base_url))
				$CSVList .= $this->Html->link($title,
							sprintf($base_url, $item['id']));
			else
				$CSVList .= $title;
			
			// the whole list is comma separated
			if ($i++ < $num_items-1)
				$CSVList .= ', ';
		}
		
		return $CSVList;
	}
	
	/** Returns an html list of values
	 * @param items An array of records
	 * @param keys An ordered array of the keys in the record that should be
	 * used to define the value. Default to the 'name' key only.
	 * @param base_url An url to view the values. Values will be linked
	 * to that url, where %d will be replaced with the value indexed by 'id'
	 * in the record. Set to null for no linking.
	 */
	public function HTMLList($items, $keys = array('name'), $base_url = null, $type = 'ul') {
		$i = 0;
		$num_items = sizeof($items);
		$num_keys = sizeof($keys);
		
		if ($num_items <= 0)
			return '';
		if ($type !== 'ul' && $type !== 'ol') {
			trigger_error("$type: No such HTML list", E_USER_WARNING);
			return '';
		}
		
		$list = "<$type>\n";
		
		foreach ($items as $item) {
			$list .= "\t<li>";
			
			// Build text to print, space separated
			$j = 0;
			$title = '';
			foreach ($keys as $key) {
				$title .= $item[$key];
				if ($j++ < $num_keys-1)
					$title .= ' ';
			}
			
			// Link if required
			if (!empty($base_url))
				$list .= $this->Html->link($title,
							sprintf($base_url, $item['id']));
			else
				$list .= $title;
			
			$list .= "</li>\n";
		}
		
		$list .= "</$type>\n";
		return $list;
	}
	
	public function seriesAlbumShortList(&$albumList) {
		if (empty($albumList))
			return ''; // Only works for arrays of at least one album
		
		$rand = rand(0, 9999);
		$list = "<ul class=\"seriesAlbumsShortList\">\n"
				."\t<li id=\"sasl{$rand}_series_{$albumList[0]['series_id']}\">\n\t\t"
				.$this->Html->link($albumList[0]['Series']['name'], '/series/view/'.$albumList[0]['series_id'])
				."\n\t\t<ul>\n";

		$seriesID = $albumList[0]['series_id'];

		foreach($albumList as &$album) {
			if ($album['series_id'] != $seriesID) {
				$list .= "\t\t</ul>\n\t</li>\n\n\t<li id=\"sasl{$rand}_series_{$album['series_id']}\">\n\t\t"
					.$this->Html->link($album['Series']['name'], '/series/view/'.$album['series_id'])."\n\t\t<ul>\n";
				$seriesID = $album['series_id'];
			}

			$list .= "\t\t\t<li>".$this->Html->link($album['title'], '/albums/view/'.$album['id'])."</li>\n";
		}
		
		$list .= "\t\t</ul>\n\t</li>\n</ul>\n";
		
		return $list;
	}
	
	public function seriesAlbumShortListJSEnabler() {
		$this->jQuery->checkInclude();
		$expand = Router::url('/'.IMAGES_URL.'plus.png');
		$expandText = __('Show more', true);
		$contract = Router::url('/'.IMAGES_URL.'minus.png');
		$contractText = __('Show less', true);
		$threshold = Configure::read('Demyo.View_albumFoldingThreshold');
		$snippet = <<<EOS
<script type="text/javascript">
//<![CDATA[
function toggle(id) {
	var self = jQuery('li#'+id);
	var expander = jQuery('> img.expander', self);
	if (expander.attr('alt') == '+') {
		expander.attr('src', '$contract');
		expander.attr('alt', '-');
		expander.attr('title', '$contractText');
		jQuery('> ul', self).slideDown();
	} else {
		expander.attr('src', '$expand');
		expander.attr('alt', '+');
		expander.attr('title', '$expandText');
		jQuery('> ul', self).slideUp();
	}
}

jQuery(function() {
	jQuery('ul.seriesAlbumsShortList > li').each(
		function() {
			var self = jQuery(this);
			if (jQuery('ul > li', self).size() > $threshold) {
				jQuery('> a', self).after(' <img src="$expand" class="expander" alt="+" title="$expandText" onclick="toggle(\''+this.id+'\')" />');
				jQuery('> ul', self).hide();
			}
		});
	});
//]]>
</script>
EOS;
		return $snippet;	
	}
}

?>