<?php
/* Copyright 2007-2009 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */

class SimplePieReaderHelper extends AppHelper {
	public $helpers = array('Text');
	
	/** Formats a SimplePie feed.
	 * 
	 * @param $feed SimplePie
	 * @param $options
	 * @return string
	 */
	public function formatFeed($feed, $options = array()) {
		$options = array_merge(array(
				'showFeedSources' => true,
				'maxEntries' => 6,
				'maxTitleCharacters' => 75,
				'maxContentCharacters' => 300
			), $options);
		
		extract($options);
		$formatted = '';
		
		$numEntries = 0;
		foreach ($feed->get_items() as $item) {
			if ($maxTitleCharacters > 0)
				$title = $this->Text->shortenWithFullWords($item->get_title(), $maxTitleCharacters);
			else
				$title =  $item->get_title();
			
			if ($maxContentCharacters > 0)
				$content = $this->Text->shortenWithFullWords($item->get_content(), $maxContentCharacters);
			else
				$content =  $item->get_title();
			
			if ($showFeedSources)
				$source = "<a href=\"{$item->get_feed()->get_link()}\" class=\"feed_title\">{$item->get_feed()->get_title()}</a>";
			else
				$source = '';
			
			if (!$item->get_date('U')) // Cannot be parsed, take it as-is
				$date = $item->get_date('');
			else
				$date = $item->get_local_date();
			if (!empty($date))
				$data = "<span class=\"feed_item_date\">$date</span>";
			else
				$date = '';
			
			$author = $item->get_author();
			if (!empty($author)) {
				$author = $author->get_name();
				if (!empty($author))
					$author = "<span class=\"feed_item_author\">$author</span>";
				else
					$author = '';
			}
			else
				$author = '';
			
			$formatted .= "<h3 class=\"feed_item_title\"><a href=\"{$item->get_link()}\">$title</a> $source</h3>\n"
				. "<div class=\"feed_item_content\">$content</div>\n"
				. "<div class=\"feed_item_meta\">$date $author</div>\n";
			
			if (++$numEntries >= $maxEntries)
				break;
		}
		
		return $formatted;
	}
}
?>
